using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace blue_dish_te
{
	internal class Dvere : MapObject
	{
		public int wallDirection = 0;
		public bool opened = false;
		public bool opening = false;

		private Texture door, lem;
		private VertexBuffer va, vb, vc, vd;

		private float doorPos = 0;
		private bool playerOnPos = false;
		private float openedTime = 0;

		public Dvere(Device dev, DatLoader dat, char type) : base(dev, dat, type)
		{
			// nacteni textury
			try
			{
				if ((door = TextureLoader.FromStream(device, dat.GetItem("textury/dvere.jpg"))) == null)
					throw new Exception("Nelze nacist texturu.");
				if ((lem = TextureLoader.FromStream(device, dat.GetItem("textury/dverelem.jpg"))) == null)
					throw new Exception("Nelze nacist texturu.");
			}
			catch(Exception e)
			{
				MessageBox.Show("Nepovedlo se nacist textury pro steny.");
				Console.WriteLine(e);
				return;
			}

			CustomVertex.PositionNormalTextured[] v;

			// vytvoreni stran dveri
			va = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) va.Lock(0, 0);

			//naplneni VB souradnicemi vrcholu             x  y    z nx ny nz  u  v
			v[0] = new CustomVertex.PositionNormalTextured(2, 0, .9f, 0, 0, -1, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(0, 0, .9f, 0, 0, -1, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(2, 3, .9f, 0, 0, -1, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(0, 0, .9f, 0, 0, -1, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(0, 3, .9f, 0, 0, -1, 0, 0);
			va.Unlock(); //odemknuti VB 

			vb = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) vb.Lock(0, 0);
			//naplneni VB souradnicemi vrcholu               x  y  z  nx ny nz  u  v
			v[0] = new CustomVertex.PositionNormalTextured(0, 0, .9f, -1, 0, 0, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(0, 0, 1.1f, -1, 0, 0, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(0, 3, .9f, -1, 0, 0, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(0, 0, 1.1f, -1, 0, 0, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(0, 3, 1.1f, -1, 0, 0, 0, 0);
			vb.Unlock(); //odemknuti VB 

			vc = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) vc.Lock(0, 0);
			//naplneni VB souradnicemi vrcholu             x  y     z nx ny nz  u  v
			v[0] = new CustomVertex.PositionNormalTextured(0, 0, 1.1f, 0, 0, 1, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(2, 0, 1.1f, 0, 0, 1, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(0, 3, 1.1f, 0, 0, 1, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(2, 0, 1.1f, 0, 0, 1, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(2, 3, 1.1f, 0, 0, 1, 0, 0);
			vc.Unlock(); //odemknuti VB 

			vd = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), 5, device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			v = (CustomVertex.PositionNormalTextured[]) vd.Lock(0, 0);
			//naplneni VB souradnicemi vrcholu                x  y  z nx ny nz  u  v
			v[0] = new CustomVertex.PositionNormalTextured(2, 0, 1.1f, 1, 0, 0, 1, 1);
			v[1] = new CustomVertex.PositionNormalTextured(2, 0, .9f, 1, 0, 0, 0, 1);
			v[2] = new CustomVertex.PositionNormalTextured(2, 3, 1.1f, 1, 0, 0, 1, 0);
			v[3] = new CustomVertex.PositionNormalTextured(2, 0, .9f, 1, 0, 0, 0, 1);
			v[4] = new CustomVertex.PositionNormalTextured(2, 3, .9f, 1, 0, 0, 0, 0);
			vd.Unlock(); //odemknuti VB 
		}

		// Rendrovaci funkce
		public override void Render()
		{
			// pokud jsou dvere otevreny spust casovac pro jejich zavreni
			if (opened)
			{
				openedTime += renderTime;
				if (openedTime > 10)
				{
					// jeli hrac na kosticce, nezavirej dvere
					if (!playerOnPos) opening = false;
					// nastav casovac na nulu
					openedTime = 0;
				}
			}

			// pokud se dvere oteviraji ci zaviraji
			if (opening && doorPos < 0.99f)
			{
				// vypocet pozice dveri
				doorPos += 2f * renderTime;
				if (doorPos > 0.99f)
				{
					doorPos = 0.99f;
					opened = true;
				}
			}
			if (!opening && doorPos > 0)
			{
				doorPos -= 2f * renderTime;
				opened = false;
				if (doorPos < 0)
					doorPos = 0;
			}

			device.VertexFormat = CustomVertex.PositionNormalTextured.Format;


			// provedeni prislusnych transformaci
			Matrix m = device.Transform.World;
			if ((wallDirection & 2) == 2 || (wallDirection & 8) == 8)
			{
				m = Matrix.Identity * Matrix.RotationY((float) Math.PI / 2) * Matrix.Translation(0, 0, 2f) * m;
				device.Transform.World = m * Matrix.Translation(0, 0, 2 * doorPos);
			}
			else
			{
				device.Transform.World = m * Matrix.Translation(2 * doorPos, 0, 0);
			}

			if (!noTexture) device.SetTexture(0, door);
			device.SetStreamSource(0, va, 0);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3);

			if (!noTexture) device.SetTexture(0, lem);
			device.SetStreamSource(0, vb, 0);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3);

			if (!noTexture) device.SetTexture(0, door);
			device.SetStreamSource(0, vc, 0);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3);

			if (!noTexture) device.SetTexture(0, lem);
			device.SetStreamSource(0, vd, 0);
			device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 3);
		}

		// volano pokud je na objekt pouzita klavea pro pouzit
		public override void UseObject()
		{
			opening = !opening;
		}
		
		// volano pri detekci kolizi
		public override bool DetectCollision()
		{
			return !opened;
		}

		// hrac prisel na kosticku
		public override bool PlayerOver()
		{
			playerOnPos = true;
			return false;
		}

		//hrac odesel z kosticky
		public override bool PlayerOut()
		{
			playerOnPos = false;
			openedTime = 0;
			return false;
		}
	}
}