using System;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Font=System.Drawing.Font;

namespace blue_dish_te
{
	//struktura pro uchovani pozice hrace
	public struct Hrac
	{
		public float x;
		public float y;
	}

	internal class Svet
	{
		//aktualni poloha hrace
		private Hrac hrac;

		// informace o mape
		public int width = 0;
		public int height = 0;

		public int startX = 0;
		public int startY = 0;

		// pokud se nema aplikovat textura, pak true
		public bool noTexture = false;

		// mapa a jeji transformace
		public char[][] map;
		private Matrix[,] mapMatrix;
		
		private Device device;

		//ktere steny se budou u 3D kosticky kreslit
		private byte[,] wallDirection;

		private MapObject[,] objects;

		//rendery
		private Stena stena = null;
		private Strop strop = null;
		private Podlaha podlaha = null;

		// muze se hrac pohybovat?
		public bool isPlayerMovable = true;

		// pivo a chipsy
		private int beer = 0;
		private int beerCounter = 0;
		private int chipsCounter = 0;
		public const int cBeerToDeath = 12;
		private LahevObject lastBeer;
		private ChipsyObject lastChips;
		private Microsoft.DirectX.Direct3D.Font beerdeath_d3dfont;
		
		// cas renderovani
		public float renderTime = 1;

		// bilicek souboru
		private DatLoader dat;
		
		private RenderForm rf;

		// timer pro padani pri smrti
		private float fallingDownTimer = 0;

		// pohyby v opilosti
		private float sousedMaxPosition = 0;
		private float sousedActualPosition = 0;
		private float sousedCycle = 0;
		private int sousedCycleDir = 0;

		//napoveda
		private bool showHelp = false;
		private Microsoft.DirectX.Direct3D.Font help_d3dfont;
		private String helpText;

		private Random rnd;

		public Svet(Device device, DatLoader dat)
		{
			ReInitialize(device, dat);
		}

		//nastaveni okna pro kresleni
		public void SetRenderForm(RenderForm rf)
		{
			this.rf = rf;
		}

		//vytvoreni objektu po resetu Device
		public void ReInitialize(Device dev, DatLoader dat)
		{
			this.dat = dat;
			stena = new Stena(dev, dat);
			strop = new Strop(dev, dat);
			podlaha = new Podlaha(dev, dat);
			rnd = new Random();
			
			beerdeath_d3dfont = new Microsoft.DirectX.Direct3D.Font(dev, new Font(FontFamily.GenericSansSerif, 30));
			help_d3dfont = new Microsoft.DirectX.Direct3D.Font(dev, new Font(FontFamily.GenericSansSerif, 10));

			device = dev;
			helpText = new StreamReader(dat.GetItem("text/help.txt"), Encoding.UTF8).ReadToEnd();
		}

		// nacteni mapy
		public bool LoadMap(string filename)
		{
			try
			{
				String sTmp;
				String[] asTmp;
				byte wallDirection;

				//otevreme soubor
				StreamReader reader = File.OpenText(filename);
				//nacteme prvni radku
				sTmp = reader.ReadLine();
				//rozparsneme na velikosti mapy
				asTmp = sTmp.Split('x');
				width = int.Parse(asTmp[0]);
				height = int.Parse(asTmp[1]);

				//vytvorime pole
				map = new char[height][];
				mapMatrix = new Matrix[height,width];
				objects = new MapObject[height,width];
				this.wallDirection = new byte[height,width];

				//nacteme mapu ze souboru
				for (int line = 0; line < height; line++)
				{
					map[line] = reader.ReadLine().ToCharArray();
				}
				//uzavreme soubor
				reader.Close();

				// Nyni projedeme celou mapu a porovedeme:
				// * hledani pocatku hrace
				// * urceni, ktere steny u 3D kosticky se budou kreslit
				// * nastaveni matice sveta na mape
				for (int x = 0; x < width; x++)
					for (int y = 0; y < height; y++)
					{
						//urceni, ktere steny u 3D kosticky se budou kreslit
						if (map[y][x] == '*' || (map[y][x] >= 'a' && map[y][x] <= 'n') || (map[y][x] >= 'A' && map[y][x] <= 'G'))
						{
							wallDirection = 0;
							if (x > 0)
								if (map[y][x - 1] != '*' && (map[y][x - 1] < 'a' || map[y][x - 1] > 'n'))
									wallDirection |= 1;
							if (x < width - 1)
								if (map[y][x + 1] != '*' && (map[y][x + 1] < 'a' || map[y][x + 1] > 'n'))
									wallDirection |= 4;
							if (y > 0)
								if (map[y - 1][x] != '*' && (map[y - 1][x] < 'a' || map[y - 1][x] > 'n'))
									wallDirection |= 2;
							if (y < height - 1)
								if (map[y + 1][x] != '*' && (map[y + 1][x] < 'a' || map[y + 1][x] > 'n'))
									wallDirection |= 8;
							if (map[y][x] == '*' || (map[y][x] >= 'a' && map[y][x] <= 'n'))
								this.wallDirection[y, x] = wallDirection;
							if (map[y][x] >= 'A' && map[y][x] <= 'G')
							{
								objects[y, x] = new Dvere(device, dat, map[y][x]);
								((Dvere) objects[y, x]).wallDirection = wallDirection;
							}
						}
						// pridani teleportu mezi objekty
						if (map[y][x] == '+')
						{
							objects[y, x] = new Teleport(device, dat, map[y][x]);
							objects[y, x].setSvet(this);
							objects[y, x].setObjectPos(x, y);
						}
						// lahev
						if (map[y][x] == 'T')
						{
							objects[y, x] = new LahevObject(device, dat, map[y][x]);
							objects[y, x].setSvet(this);
							objects[y, x].setObjectPos(x, y);
							lastBeer = (LahevObject) objects[y, x];
						}
						// chipsy
						if (map[y][x] == 'U')
						{
							objects[y, x] = new ChipsyObject(device, dat, map[y][x]);
							objects[y, x].setSvet(this);
							objects[y, x].setObjectPos(x, y);
							lastChips = (ChipsyObject) objects[y, x];
						}

						//hledani pocatku hrace
						if (map[y][x] == 'S')
						{
							startX = y;
							startY = x;
						}

						//nastaveni matice sveta na mape
						mapMatrix[y, x] = Matrix.Identity * Matrix.Translation(y * 2f, 0f, x * 2f);
					}

				//nastaveni vychozich hodnot
				ResetObjectParams();

				// nastaveni hrace na start pozici
				setHrac(-(startX * 2 + 1), -(startY * 2 + 1), true);
				return true;
			}
			catch(Exception e)
			{
				MessageBox.Show("Nepovedlo se nacist mapu.");
				Console.WriteLine(e);
				return false;
			}
		}
		
		// volani kolizi pro objekt
		public bool KolizeObjects(int x, int y)
		{
			if (objects[x, y] != null)
				return objects[x, y].DetectCollision();
			else
				return false;
		}

		// volani pouziti objektu
		public void UseObject(int x, int y)
		{
			if (objects[x, y] != null) objects[x, y].UseObject();
		}

		// vraci aktualni pozici hrace
		public Hrac getHrac()
		{
			return hrac;
		}

		//pretizene funkce
		public void setHrac(Hrac hrac, bool noEvents)
		{
			setHrac(hrac.x, hrac.y, noEvents);
		}

		public void setHrac(Hrac hrac)
		{
			setHrac(hrac.x, hrac.y, false);
		}

		public void setHrac(float x, float y)
		{
			setHrac(x, y, false);
		}

		// nastavuje novou pozici hrace, pripadne parametr, pokud se nemaj vykonat eventy
		public void setHrac(float x, float y, bool noEvents)
		{
			bool ret = false;

			if (!noEvents && isPlayerMovable)
			{
				if ((int) (-hrac.x / 2) != (int) (-x / 2) || (int) (-hrac.y / 2) != (int) (-y / 2))
				{
					if (objects[(int) -hrac.x / 2, (int) -hrac.y / 2] != null)
						ret |= objects[(int) -hrac.x / 2, (int) -hrac.y / 2].PlayerOut();

					if (objects[(int) -x / 2, (int) -y / 2] != null)
						ret |= objects[(int) -x / 2, (int) -y / 2].PlayerOver();

					if (ret)
						return;
				}
			}
			hrac.x = x;
			hrac.y = y;
		}

		// nastaveni startovaci pozice
		public void setHracHP()
		{
			ResetObjectParams();
			setHrac(-(startX * 2 + 1), -(startY * 2 + 1));
		}

		// renderovaci funkce
		public void Render()
		{
			stena.noTexture = noTexture;
			strop.noTexture = noTexture;
			podlaha.noTexture = noTexture;

			//renderujeme mapu
			for (int x = 0; x < width; x++)
			{
				for (int y = 0; y < height; y++)
				{
					//pokud se jedna o zed
					device.Transform.World = mapMatrix[y, x];
					if (map[y][x] == '*' || (map[y][x] >= 'a' && map[y][x] <= 'n'))
					{
						//typ steny
						stena.type = map[y][x];
						stena.wallDirection = wallDirection[y, x];
						stena.Render();
					}
					// pokud to neni stena renderujeme strop a podlahu
					if (map[y][x] != '*' && (map[y][x] < 'a' || map[y][x] > 'n'))
					{
						//typ stropu
						strop.type = map[y][x];
						strop.noTexture = noTexture;
						strop.Render();
						//typ podlahy
						podlaha.type = map[y][x];
						podlaha.Render();
					}
					// rendrovat objekt pokud je nastaven
					if (objects[y, x] != null)
					{
						objects[y, x].renderTime = renderTime;
						objects[y, x].noTexture = noTexture;
						objects[y, x].Render();
					}
				}
			}
		}

		// pridani vypiteho piva
		public void beerAdd()
		{
			if (beer < cBeerToDeath)
			{
				beer++;
				beerCounter++;
			}
			if (beer >= cBeerToDeath) isPlayerMovable = false;
		}

		// odebrani piva
		public void beerRemove()
		{
			if (beer > 0) beer--;
			chipsCounter++;
		}

		// vraceni poctu vypitych piv
		public int beerGet()
		{
			return beer;
		}

		// nastaveni vychozich hodnot
		public void ResetObjectParams()
		{
			beer = 0;
			isPlayerMovable = true;
		}

		// kresleno po hlavni renderaci
		public void postRender()
		{
			// kreslime texty
			StringBuilder text = new StringBuilder();
			text.AppendLine();
			text.AppendLine("Sebrno:");
			text.AppendFormat("  Pivo: {0}", beerCounter);
			text.AppendLine();
			text.AppendFormat("  Chipsy: {0}", chipsCounter);
			text.AppendLine();
			int soused = ((int) (((float) beer / cBeerToDeath) * 100));
			if (soused < 0) soused = 0;

			text.AppendFormat("Opilost: {0}%", soused);

			rf.drawText(null, text.ToString(), rf.ClientRectangle, DrawTextFormat.Top | DrawTextFormat.Left | DrawTextFormat.WordBreak, Color.Yellow);
			rf.drawText(null, "Klvesa Tab pro help", rf.ClientRectangle, DrawTextFormat.Bottom | DrawTextFormat.Right | DrawTextFormat.WordBreak, Color.Yellow);
			// pokud toho moc vypil, tak zobrazit smrtici hlasku
			if (beer >= cBeerToDeath)
			{
				beerdeath_d3dfont.DrawText(null, "Jsi mrtvej!\nPli mnoho alkoholu pro tvoje tlo!\nZmkni klvesu H nebo R pro resuscitaci...", rf.ClientRectangle, DrawTextFormat.Center | DrawTextFormat.VerticalCenter, Color.Red);
				device.Transform.View = device.Transform.View * Matrix.RotationZ((float) -Math.PI / 2);
			}

			// zobrazeni helpu
			if (showHelp)
			{
				help_d3dfont.DrawText(null, helpText, new Rectangle((rf.Width - 260) / 2, (rf.Height - 350) / 2, 260, 350), DrawTextFormat.Top | DrawTextFormat.Left | DrawTextFormat.WordBreak, Color.White);
			}
		}

		// volano pred hlavni renderaci
		public void preRender()
		{
			const float cSousedCycles = 0.3f;

			//pokud jsi vypita piva, proved pocitani pohybu opilosti
			if (beer > 0)
			{
				sousedMaxPosition = ((float) beer / cBeerToDeath) * ((float) Math.PI / 20);

				if (sousedCycle > cSousedCycles)
				{
					sousedCycle = 0;
					sousedCycleDir = rnd.Next(-5, 5);
					if (rnd.Next(1, 100) % 2 == 1)
						sousedCycleDir = -sousedCycleDir;
				}
				float newval = sousedActualPosition + sousedCycleDir * ((float) beer / (cBeerToDeath)) * (renderTime * cSousedCycles) * ((float) Math.PI / 20);
				if (Math.Abs(newval) > sousedMaxPosition)
				{
					sousedCycleDir = -sousedCycleDir;
				}
				sousedActualPosition += sousedCycleDir * ((float) beer / (cBeerToDeath)) * (renderTime * cSousedCycles) * ((float) Math.PI / 20);
				sousedCycle += renderTime;
			}
			else
			{
				sousedMaxPosition = 0;
				sousedActualPosition = 0;
				sousedCycleDir = 0;
				sousedCycle = 0;
			}

			// pokud toho moc vypil, proved pad hrace
			if (beer >= cBeerToDeath)
			{
				sousedMaxPosition = 0;
				sousedActualPosition = 0;
				sousedCycleDir = 0;
				sousedCycle = 0;
				if (fallingDownTimer < 1f)
					fallingDownTimer += renderTime;
				else
					fallingDownTimer = 1f;
				float par = (float) Math.Pow(16, fallingDownTimer) / 16;
				device.Transform.View = Matrix.Translation(0, par * 1.5f, 0) * device.Transform.View * Matrix.RotationZ((par * (float) -Math.PI) / 2);
			}
			else if (beer > 0)
			{
				device.Transform.View = device.Transform.View * Matrix.Translation(0, 1.85f, 0) * Matrix.RotationZ((float) -Math.Sin(sousedActualPosition) * ((float) -Math.PI / 2)) * Matrix.Translation(0, -1.85f, 0);
				fallingDownTimer = 0;
			}
			else
			{
				fallingDownTimer = 0;
			}
		}

		// zapnuti vypnuti napovedy
		public void ToggleHelp()
		{
			showHelp = !showHelp;
		}
	}
}